/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.santander.web;

import java.rmi.RemoteException;
import java.util.Date;

import org.apache.commons.lang3.tuple.Pair;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;

import de.frankmuenster.mahoe.banks.santander.SantanderBank;
import de.frankmuenster.mahoe.banks.santander.webpages.SantanderPostBoxPage;
import de.frankmuenster.mahoe.messages.AbstractBankMessages;
import de.frankmuenster.mahoe.messages.MahoeMessage;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.webpages.PostBoxPage;
import de.willuhn.util.ApplicationException;

/**
 * Bank-Nachrichten der BOS
 *
 * @author Frank Mnster
 */
public class SantanderBankMessages extends AbstractBankMessages<WebDriver, SantanderWebAccountInformation> {

    /**
     * Number of title elements in messages
     */
    private static final int MSG_TITLE_ELEMENTS_NUMBER = 2;

    /**
     * @param webClient
     * @param accountinfo
     */
    public SantanderBankMessages(final WebDriver webClient, final SantanderWebAccountInformation accountinfo) {
        super(webClient, accountinfo);

    }

    /**
     * @see de.frankmuenster.mahoe.messages.InterfaceBankMessages#markAsRead(de.frankmuenster.mahoe.messages.MahoeMessage)
     */
    @Override
    public void markAsRead(final MahoeMessage message) throws ApplicationException {

        // Not needed as the user still needs to download the message
    }

    /**
     * @see de.frankmuenster.mahoe.messages.AbstractBankMessages#readMessages(java.util.Date)
     */
    @Override
    public void readMessages(final Date startDate) throws WebSiteException, ApplicationException {

        // load transactions from PDFs
        try {
            final PostBoxPage<WebElement> postBoxPage = new SantanderPostBoxPage(this.webclient, this.accountInfo);

            // create StringBufr to be reused in loop
            final StringBuilder msgText = new StringBuilder(150);

            while (postBoxPage.hasNextDocument()) {

                // get the document
                final Pair<Date, WebElement> msgLink = postBoxPage.nextDocument();

                // the key is the date
                final Date msgDate = msgLink.getKey();

                // the value has the WebElement with the text inside
                final String[] titles = msgLink.getValue().getText().split("\n"); //$NON-NLS-1$
                String title = msgLink.getValue().getText();
                if (titles.length == SantanderBankMessages.MSG_TITLE_ELEMENTS_NUMBER) {
                    title = titles[1];
                }

                // TODO extract message from PDF
                msgText.setLength(0);
                msgText.append(msgLink.getValue().getText())
                        .append("\n\nBitte laden Sie diesen ber die Santander Webseite herunter.\n"); //$NON-NLS-1$

                // create the new message element
                final MahoeMessage msg = new MahoeMessage(SantanderBank.BLZ, msgDate, title, // NOPMD
                        msgText.toString());

                // and finally add it, if it not exists already
                addMessage(msg);

            }

        } catch (final RemoteException e) {
            throw new ApplicationException(e);
        }

    }

}
