/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.targobank.web;

import java.io.IOException;
import java.io.StringReader;
import java.util.Date;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.lang3.tuple.Pair;
import org.openqa.selenium.WebDriver;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import de.frankmuenster.mahoe.banks.targobank.TargoBank;
import de.frankmuenster.mahoe.banks.targobank.web.webpages.TargoBankPostBoxPage;
import de.frankmuenster.mahoe.messages.AbstractBankMessages;
import de.frankmuenster.mahoe.messages.MahoeMessage;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.frankmuenster.mahoe.web.webpages.PostBoxPage;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * List und Speichert Bank-Nachrichten
 *
 * @author Frank Mnster
 */
public class TargoBankMessages extends AbstractBankMessages<WebDriver, TargoBankWebAccountInformation> {

    /**
     *
     */
    private static final String NO_MESSAGES = "Sie haben keine Mitteilung."; //$NON-NLS-1$ //NOPMD

    /**
     * Erstellt ein neues Objekt
     *
     * @param webclient
     * @param accountInfo
     */
    public TargoBankMessages(final WebDriver webClient, final TargoBankWebAccountInformation accountInfo) {
        super(webClient, accountInfo);
    }

    /**
     * @see de.frankmuenster.mahoe.messages.InterfaceBankMessages#markAsRead(de.frankmuenster.mahoe.messages.MahoeMessage)
     *
     * @param message
     */
    @Override
    public void markAsRead(final MahoeMessage message) throws ApplicationException {
        // Wir bei der TargoBank nicht bentigt, da wir auf den Link 'Klicken'.

    }

    /**
     * @see de.frankmuenster.mahoe.messages.AbstractBankMessages#readMessages(java.util.Date)
     * @throws WebSiteException
     * @throws ApplicationException
     */
    @Override
    public void readMessages(final Date startDate) throws WebSiteException, ApplicationException {

        DocumentBuilder dBuilder;
        try {
            // create the DOM document builder outside loop
            final DocumentBuilderFactory dBuilderFactory = DocumentBuilderFactory.newInstance();
            dBuilder = dBuilderFactory.newDocumentBuilder();
        } catch (final ParserConfigurationException e) {
            throw new WebSiteException("Could not parse document link data.", WebSiteExceptionTypes.PARSE_LINK_ERROR,
                    e);

        }

        // create StringBufr to be reused in loop
        final StringBuilder msgText = new StringBuilder(150);

        // load transactions from PDFs
        final PostBoxPage<String> postBoxPage = TargoBankPostBoxPage.getInstance(this.webclient, this.accountInfo);

        // ensure that we read from the beginning
        postBoxPage.resetDocumentsReadPosition();

        // looping over all messages
        while (postBoxPage.hasNextDocument()) {
            try {

                // get the document
                final Pair<Date, String> msgLink = postBoxPage.nextDocument();

                // the key is the date
                final Date msgDate = msgLink.getKey();

                // weiter falls before startDate
                if (msgDate.before(startDate)) {
                    continue;
                }

                // Extract information
                final String value = msgLink.getValue();
                final Document document = dBuilder.parse(new InputSource(new StringReader(value))); // NOPMD
                final Element root = document.getDocumentElement();
                final NodeList aLink = root.getElementsByTagName("a"); //$NON-NLS-1$
                final String title = aLink.item(0).getTextContent();
                Logger.debug("Document Titel: " + title);

                // TODO: extract text from PDF

                msgText.setLength(0);
                msgText.append(title).append("\n\nBitte laden Sie diesen ber die Targo Webseite herunter.\n"); //$NON-NLS-1$

                // create the new message element
                final MahoeMessage msg = new MahoeMessage(TargoBank.BLZ, msgDate, title, // NOPMD
                        msgText.toString());

                // and finally add it, if it not exists already
                addMessage(msg);

            } catch (SAXException | IOException e) {
                throw new WebSiteException("Could not parse document link data.",
                        WebSiteExceptionTypes.PARSE_LINK_ERROR, e);
            }
        }
    }
}
