
package de.frankmuenster.mahoe.web.webpages;

import java.util.Date;
import java.util.NoSuchElementException;

import org.apache.commons.lang3.tuple.Pair;

/**
 * Interface for a PostBax page
 *
 * T is either a WebElement or a String with the content of the WebElement
 *
 * @author Frank Mnster
 * @since 2.6.0
 *
 */
public interface PostBoxPage<T> {

    /**
     * Returns {@code true} if the PostBox has more accounts sheets/card
     * billings. (In other words, returns {@code true} if {@link #nextDocument}
     * would return an element rather than throwing an exception.)
     *
     * @return {@code true} if the PostBox has more elements
     */
    boolean hasNextAccountSheet();

    /**
     * Returns {@code true} if the PostBox has more accounts sheets/card
     * billings. (In other words, returns {@code true} if {@link #nextDocument}
     * would return an element rather than throwing an exception.)
     *
     * @return {@code true} if the PostBox has more elements
     */
    boolean hasNextDocument();

    /**
     * Returns the next accounts sheet/card billing from the PostBox
     *
     * @return the next accounts sheets/card billings as Pair< Date,WebElement >
     * @throws NoSuchElementException
     *             - if the PostBox has no more elements
     *
     */
    Pair<Date, T> nextAccountSheet() throws NoSuchElementException;

    /**
     * Returns the next document from the PostBox
     *
     * @return the next document as Pair< Date,WebElement >
     * @throws NoSuchElementException
     *             - if the PostBox has no more elements
     *
     */
    Pair<Date, T> nextDocument() throws NoSuchElementException;

    /**
     * Set accountSheet read position back to 0 beginning
     */
    void resetAccountSheetReadPosition();

    /**
     * Set documents read position back to 0 beginning
     */
    void resetDocumentsReadPosition();

}